@@ -56,6 +56,16 @@ class TestCollect(testtools.TestCase):
                              'cfn': test_cfn.FakeRequests(self)}
         collect.__main__(args=fake_args, requests_impl_map=requests_impl_map)
 
+    def _fake_popen_call_main(self, occ_args):
+        calls = []
+
+        def capture_popen(proc_args):
+            calls.append(proc_args)
+            return dict(returncode=0)
+        self.useFixture(fixtures.FakePopen(capture_popen))
+        self._call_main(occ_args)
+        return calls
+
     def test_main(self):
         expected_cmd = self.getUniqueString()
         cache_dir = self.useFixture(fixtures.TempDir())
@@ -81,13 +91,7 @@ class TestCollect(testtools.TestCase):
             '--heat_local-path',
             fake_metadata,
         ]
-        calls = []
-
-        def capture_popen(proc_args):
-            calls.append(proc_args)
-            return dict(returncode=0)
-        self.useFixture(fixtures.FakePopen(capture_popen))
-        self._call_main(occ_args)
+        calls = self._fake_popen_call_main(occ_args)
         proc_args = calls[0]
         self.assertEqual(expected_cmd, proc_args['args'])
         list_path = os.path.join(cache_dir.path, 'os_config_files.json')
@@ -109,6 +113,24 @@ class TestCollect(testtools.TestCase):
         self.assertIn("int1", keys_found)
         self.assertIn("map_ab", keys_found)
 
+    def test_main_force_command(self):
+        cache_dir = self.useFixture(fixtures.TempDir())
+        fake_metadata = _setup_local_metadata(self)
+        occ_args = [
+            'os-collect-config',
+            '--command', 'foo',
+            '--cachedir', cache_dir.path,
+            '--config-file', '/dev/null',
+            '--heat_local-path', fake_metadata,
+            '--force',
+        ]
+        calls = self._fake_popen_call_main(occ_args)
+        self.assertIn('OS_CONFIG_FILES', calls[0]['env'])
+        cfg.CONF.reset()
+        # First time caches data, run again, make sure we run command again
+        calls = self._fake_popen_call_main(occ_args)
+        self.assertIn('OS_CONFIG_FILES', calls[0]['env'])
+
     def test_main_command_failed_no_caching(self):
         cache_dir = self.useFixture(fixtures.TempDir())
         fake_metadata = _setup_local_metadata(self)
@@ -170,6 +192,7 @@ class TestCollect(testtools.TestCase):
             '--config-file', '/dev/null',
             '--print-cachedir',
         ]
+
         output = self.useFixture(fixtures.StringStream('stdout'))
         self.useFixture(
             fixtures.MonkeyPatch('sys.stdout', output.stream))
@@ -177,6 +200,41 @@ class TestCollect(testtools.TestCase):
         cache_dir = output.getDetails()['stdout'].as_text().strip()
         self.assertEquals(fake_cachedir.path, cache_dir)
 
+    def test_main_print_only(self):
+        cache_dir = self.useFixture(fixtures.TempDir())
+        fake_metadata = _setup_local_metadata(self)
+        args = [
+            'os-collect-config',
+            '--command', 'bar',
+            '--cachedir', cache_dir.path,
+            '--config-file', '/dev/null',
+            '--print',
+            '--cfn-metadata-url',
+            'http://127.0.0.1:8000/v1/',
+            '--cfn-stack-name',
+            'foo',
+            '--cfn-path',
+            'foo.Metadata',
+            '--cfn-access-key-id',
+            '0123456789ABCDEF',
+            '--cfn-secret-access-key',
+            'FEDCBA9876543210',
+            '--heat_local-path', fake_metadata,
+        ]
+
+        def fake_popen(args):
+            self.fail('Called command instead of printing')
+        self.useFixture(fixtures.FakePopen(fake_popen))
+        output = self.useFixture(fixtures.StringStream('stdout'))
+        self.useFixture(
+            fixtures.MonkeyPatch('sys.stdout', output.stream))
+        self._call_main(args)
+        out_struct = json.loads(output.getDetails()['stdout'].as_text())
+        self.assertThat(out_struct, matchers.IsInstance(dict))
+        self.assertIn('cfn', out_struct)
+        self.assertIn('heat_local', out_struct)
+        self.assertIn('ec2', out_struct)
+
     def test_main_invalid_collector(self):
         fake_args = ['os-collect-config', 'invalid']
         self.assertRaises(exc.InvalidArguments, self._call_main, fake_args)
